import logging
import os

import pytest
from dagster_shell.utils import execute, execute_script_file


def test_bad_output_logging():
    with pytest.raises(Exception, match='Unrecognized output_logging NOT_A_VALID_LOGGING_VALUE'):
        execute('ls', output_logging='NOT_A_VALID_LOGGING_VALUE', log=logging)


def test_execute_inline(tmp_file):
    with tmp_file('some file contents') as (tmp_path, tmp_file):
        res, retcode = execute('ls', cwd=tmp_path, output_logging='BUFFER', log=logging)
        assert os.path.basename(tmp_file) in res
        assert retcode == 0


def test_execute_file(tmp_file):
    with tmp_file('ls') as (tmp_path, tmp_file):
        res, retcode = execute_script_file(
            tmp_file, output_logging='BUFFER', log=logging, cwd=tmp_path
        )
        assert os.path.basename(tmp_file) in res
        assert retcode == 0


def test_env(tmp_file):
    cmd = 'echo $TEST_VAR'
    res, retcode = execute(
        cmd, output_logging='BUFFER', log=logging, env={'TEST_VAR': 'some_env_value'}
    )
    assert res.strip() == 'some_env_value'
    assert retcode == 0

    with tmp_file(cmd) as (_, tmp_file):
        res, retcode = execute_script_file(
            tmp_file, output_logging='BUFFER', log=logging, env={'TEST_VAR': 'some_env_value'},
        )
        assert res.strip() == 'some_env_value'
        assert retcode == 0


def test_output_logging_stream(caplog):
    caplog.set_level(logging.INFO)

    _, retcode = execute('ls', output_logging='STREAM', log=logging)
    log_messages = [r.message for r in caplog.records]
    assert log_messages[0].startswith('Using temporary directory: ')
    assert log_messages[1].startswith('Temporary script location: ')
    assert log_messages[2] == 'Running command:\nls'
    assert log_messages[3]
    assert retcode == 0

    caplog.clear()

    _, retcode = execute('ls', output_logging='STREAM', log=logging)
    log_messages = [r.message for r in caplog.records]
    assert log_messages[0].startswith('Using temporary directory: ')
    assert log_messages[1].startswith('Temporary script location: ')
    assert log_messages[2] == 'Running command:\nls'
    assert log_messages[3]
    assert retcode == 0

    caplog.clear()

    _, retcode = execute(
        'for i in 1 2 3; do echo "iter $i"; done;', output_logging='STREAM', log=logging,
    )
    log_messages = [r.message for r in caplog.records]
    assert retcode == 0
    assert log_messages[3:6] == ['iter 1', 'iter 2', 'iter 3']

    caplog.clear()

    _, retcode = execute(
        'for i in 1 2 3; do echo "iter $i"; done;', output_logging='BUFFER', log=logging,
    )
    log_messages = [r.message for r in caplog.records]
    assert retcode == 0
    assert log_messages[3] == 'iter 1\niter 2\niter 3\n'



import os

import pytest
from dagster_shell import create_shell_command_solid, create_shell_script_solid, shell_solid

from dagster import Failure, OutputDefinition, composite_solid, execute_solid


def test_shell_command_solid():
    solid = create_shell_command_solid('echo "this is a test message: $MY_ENV_VAR"', name='foobar')

    result = execute_solid(
        solid, run_config={'solids': {'foobar': {'config': {'env': {'MY_ENV_VAR': 'foobar'}}}}},
    )
    assert result.output_values == {'result': 'this is a test message: foobar\n'}


def test_shell_solid():

    result = execute_solid(
        shell_solid,
        input_values={'shell_command': 'echo "this is a test message: $MY_ENV_VAR"'},
        run_config={'solids': {'shell_solid': {'config': {'env': {'MY_ENV_VAR': 'foobar'}}}}},
    )
    assert result.output_values == {'result': 'this is a test message: foobar\n'}


def test_shell_command_retcode():
    with pytest.raises(Failure, match='Shell command execution failed'):
        execute_solid(create_shell_command_solid('exit 1', name='exit_solid'))


def test_shell_solid_retcode():
    with pytest.raises(Failure, match='Shell command execution failed'):
        execute_solid(shell_solid, input_values={'shell_command': 'exit 1'})


def test_shell_command_stream_logs():
    solid = create_shell_command_solid(
        'for i in 1 2 3 4 5; do echo "hello ${i}"; done', name='foobar'
    )

    result = execute_solid(
        solid,
        run_config={
            'solids': {
                'foobar': {'config': {'output_logging': 'STREAM', 'env': {'MY_ENV_VAR': 'foobar'}}}
            }
        },
    )
    assert result.output_values == {'result': 'hello 1\nhello 2\nhello 3\nhello 4\nhello 5\n'}


def test_shell_script_solid():
    script_dir = os.path.dirname(os.path.abspath(__file__))
    solid = create_shell_script_solid(os.path.join(script_dir, 'test.sh'), name='foobar')
    result = execute_solid(
        solid, run_config={'solids': {'foobar': {'config': {'env': {'MY_ENV_VAR': 'foobar'}}}}},
    )
    assert result.output_values == {'result': 'this is a test message: foobar\n'}


def test_shell_script_solid_no_config():
    script_dir = os.path.dirname(os.path.abspath(__file__))
    solid = create_shell_script_solid(os.path.join(script_dir, 'test.sh'), name='foobar')
    result = execute_solid(solid)
    assert result.output_values == {'result': 'this is a test message: \n'}


def test_shell_script_solid_no_config_composite():
    script_dir = os.path.dirname(os.path.abspath(__file__))
    solid = create_shell_script_solid(os.path.join(script_dir, 'test.sh'), name='foobar')

    @composite_solid(
        config_schema={}, config_fn=lambda cfg: {}, output_defs=[OutputDefinition(str, 'result')]
    )
    def composite():
        return solid()

    result = execute_solid(composite)
    assert result.output_values == {'result': 'this is a test message: \n'}


def test_shell_command_solid_overrides():
    solid = create_shell_command_solid(
        'echo "this is a test message: $MY_ENV_VAR"',
        name='foobar',
        description='a description override',
    )

    result = execute_solid(
        solid, run_config={'solids': {'foobar': {'config': {'env': {'MY_ENV_VAR': 'foobar'}}}}},
    )
    assert result.output_values == {'result': 'this is a test message: foobar\n'}



from dagster_shell import create_shell_script_solid

from dagster import file_relative_path, pipeline


@pipeline
def pipe():
    a = create_shell_script_solid(file_relative_path(__file__, 'hello_world.sh'), name='a')
    a()



from dagster.core.utils import check_dagster_package_version

from .solids import create_shell_command_solid, create_shell_script_solid, shell_solid
from .version import __version__

check_dagster_package_version('dagster-shell', __version__)

__all__ = ['create_shell_command_solid', 'create_shell_script_solid', 'shell_solid']
